# -*- coding: utf-8 -*-
# Generated by Django 1.11.3 on 2017-07-10 06:57
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('heavy_celery', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Worker',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('worker_id', models.CharField(max_length=256, verbose_name='\u30ef\u30fc\u30ab\u30fcID')),
                ('started_at', models.DateTimeField(auto_now_add=True)),
                ('ended_at', models.DateTimeField(blank=True, null=True)),
                ('beated_at', models.DateTimeField(blank=True, null=True)),
                ('status', models.IntegerField(choices=[(0, '\u958b\u59cb'), (1, '\u505c\u6b62'), (2, '\u5fdc\u7b54\u306a\u3057')], default=0)),
            ],
        ),
        migrations.CreateModel(
            name='WorkerTaskLog',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('task_id', models.CharField(max_length=256, unique=True)),
                ('task_path', models.CharField(max_length=256)),
                ('started_at', models.DateTimeField(auto_now_add=True)),
                ('ended_at', models.DateTimeField(blank=True, null=True)),
                ('worker', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='heavy_celery.Worker')),
            ],
        ),
    ]
